/*==========================================================================
 *  OpenGL Fullscreen/Window mode example
 *
 *  Version 1.00
 *
 * Copyright (C) Allan Petersen.  All Rights Reserved.
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that
 * the name of Allan Petersen not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission.
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL ALLAN
 * PETERSEN BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT ALLAN PETERSEN HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *--------------------------------------------------------------------------
 *
 * Contact Allan Petersen at <allan.petersen@danbbs.dk> or visit
 * www.danbbs.dk/~allmette
 * www.allanpetersen.com
 *
 *==========================================================================
 */
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include <math.h>
#include <stdio.h>
#include "Unit1.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "TOpenGLAPPanel"
#pragma resource "*.dfm"
TForm1 *Form1;

float points[45][45][3];    // The Array For The Points On The Grid Of Our "Wave"
int wiggle_count = 0;		// Counter Used To Control How Fast Flag Waves

GLfloat	xrot;				// X Rotation ( NEW )
GLfloat	yrot;				// Y Rotation ( NEW )
GLfloat	zrot;				// Z Rotation ( NEW )
GLfloat hold;				// Temporarily Holds A Floating Point Value

GLuint	texture[1];			// Storage For One Texture ( NEW )

//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner)
{
    DEVMODE DevMode;    // ---- Device mode
    int     rc;         // ---- Result code
    int     xmode;      // ---- Index to enumeration list


    // ---- List display enmunerations until fault.
    xmode = 0;
    do  {
        // ---- Enumerate available display settings
        rc = EnumDisplaySettings(NULL, xmode++, &DevMode);
        if (rc != 0) {
            // ---- Add resultion ti list
            ComboBoxResolutions->Items->Add (IntToStr(DevMode.dmPelsWidth)  + "X" +
                                             IntToStr(DevMode.dmPelsHeight) + "X" +
                                             IntToStr(DevMode.dmDisplayFrequency) + "Hz " +
                                             IntToStr(DevMode.dmBitsPerPel) + "-Bit");
            }
        } while (rc != 0);
}
//---------------------------------------------------------------------------
int __fastcall  TForm1::LoadGLTextures() // Load Bitmaps And Convert To Textures
{
	int Status=FALSE;									// Status Indicator
    BMPTexture  *BMPTex = NULL;

	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
    BMPTex = OpenGLAPPanel1->LoadBMPTexture ("Data/ap.bmp", false);
    if (BMPTex)	{
		Status=TRUE;									// Set The Status To TRUE

		glGenTextures(1, &texture[0]);					// Create The Texture

		// Typical Texture Generation Using Data From The Bitmap
		glBindTexture(GL_TEXTURE_2D, texture[0]);
		glTexImage2D(GL_TEXTURE_2D, 0, 3, BMPTex->GetWidth(), BMPTex->GetHeight(), 0, GL_RGB, GL_UNSIGNED_BYTE, BMPTex->GetPixels());
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        delete BMPTex;
    	}
	return Status;										// Return The Status
}
//---------------------------------------------------------------------------
void __fastcall TForm1::OpenGLAPPanel1Init(TObject *Sender)
{
    float  float_x;
    float  float_y;

	if (!LoadGLTextures()) {							// Jump To Texture Loading Routine
		return;									        // If Texture Didn't Load Return FALSE
	    }
	glEnable(GL_TEXTURE_2D);							// Enable Texture Mapping ( NEW )
	glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);				// Black Background
	glClearDepth(1.0f);									// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);								// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations
	glPolygonMode( GL_BACK, GL_FILL );					// Back Face Is Solid
	glPolygonMode( GL_FRONT, GL_LINE );					// Front Face Is Made Of Lines

	for(float_x = 0.0f; float_x < 9.0f; float_x +=  0.2f )	{
		for(float_y = 0.0f; float_y < 9.0f; float_y += 0.2f)		{
			points[ int(float_x*5.0f) ][ int(float_y*5.0f) ][0] = float_x - 4.4f;
			points[ int(float_x*5.0f) ][ int(float_y*5.0f) ][1] = float_y - 4.4f;
			points[ int(float_x*5.0f) ][ int(float_y*5.0f) ][2] = float(sin( ( (float_x*40.0f)/360.0f) * 3.14159f * 2.0f ));
		    }
	    }

}
//---------------------------------------------------------------------------
void __fastcall TForm1::OpenGLAPPanel1Paint(TObject *Sender)
{
	int x, y;
	float float_x, float_y, float_xb, float_yb;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear The Screen And The Depth Buffer
	glLoadIdentity();									// Reset The View

	glTranslatef(0.0f,0.0f,-12.0f);
	  
	glRotatef(xrot,1.0f,0.0f,0.0f);
	glRotatef(yrot,0.0f,1.0f,0.0f);  
	glRotatef(zrot,0.0f,0.0f,1.0f);

	glBindTexture(GL_TEXTURE_2D, texture[0]);

	glBegin(GL_QUADS);
	for( x = 0; x < 44; x++ ) {
		for( y = 0; y < 44; y++ ) {
			float_x = float(x)/44.0f;
			float_y = float(y)/44.0f;
			float_xb = float(x+1)/44.0f;
			float_yb = float(y+1)/44.0f;

			glTexCoord2f( float_x, float_y);
			glVertex3f( points[x][y][0], points[x][y][1], points[x][y][2] );

			glTexCoord2f( float_x, float_yb );
			glVertex3f( points[x][y+1][0], points[x][y+1][1], points[x][y+1][2] );

			glTexCoord2f( float_xb, float_yb );
			glVertex3f( points[x+1][y+1][0], points[x+1][y+1][1], points[x+1][y+1][2] );

			glTexCoord2f( float_xb, float_y );
			glVertex3f( points[x+1][y][0], points[x+1][y][1], points[x+1][y][2] );
		    }
	    }
	glEnd();

	if( wiggle_count == 2 )	{
		for( y = 0; y < 45; y++ ) {
			hold=points[0][y][2];
			for( x = 0; x < 44; x++) {
				points[x][y][2] = points[x+1][y][2];
			    }
			points[44][y][2]=hold;
		    }
		wiggle_count = 0;
	    }

	wiggle_count++;

	xrot+=0.3f;
	yrot+=0.2f;
	zrot+=0.4f;

}
//---------------------------------------------------------------------------
void __fastcall TForm1::OpenGLAPPanel1Resize(TObject *Sender)
{
	if (OpenGLAPPanel1->Height==0) {  							// Prevent A Divide By Zero By
		OpenGLAPPanel1->Height=1;										// Making Height Equal One
	    }

	glViewport(0,0,OpenGLAPPanel1->Width,OpenGLAPPanel1->Height);						// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(45.0f,(GLfloat)OpenGLAPPanel1->Width/(GLfloat)OpenGLAPPanel1->Height,0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Timer1Timer(TObject *Sender)
{
    OpenGLAPPanel1->Repaint ();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ButtonTglResolutionClick(TObject *Sender)
{
    DEVMODE DevMode;
    int     rc, xmode;

    // ---- Using the developer tag to save fulscreen/window state
    switch (ButtonTglResolution->Tag) {
        // ---- Going to fullscreen
        case 0:
            // ---- Verify user selction
            if (ComboBoxResolutions->ItemIndex == -1) return;

            // ---- Saving current position of window
            GetWindowPlacement(Handle, &WindowRect);

            // ---- Get Device Mode data
            EnumDisplaySettings(NULL, ComboBoxResolutions->ItemIndex, &DevMode);

            // ---- First Test resolution selected
            rc = ChangeDisplaySettings(&DevMode, CDS_TEST);
            switch (rc) {
                case DISP_CHANGE_SUCCESSFUL:
                    // ----  The settings change was successful.
                    ChangeDisplaySettings(&DevMode, CDS_FULLSCREEN);
                    BorderStyle =  bsNone;      // ---- Removing window border
                    WindowState = wsMaximized;  // ---- Maximizing window
                    ButtonTglResolution->Caption = "Window";
                    break;
                case DISP_CHANGE_RESTART:
                    ChangeDisplaySettings(&DevMode, CDS_FULLSCREEN);
                    BorderStyle =  bsNone;      // ---- Removing window border
                    WindowState = wsMaximized;  // ---- Maximizing window
                    ButtonTglResolution->Caption = "Window";
                    MessageBox(NULL,"The computer must be restarted in order for the graphics mode to work.","Restart",MB_OK|MB_ICONSTOP);
                    break;
                case DISP_CHANGE_BADFLAGS:
                    MessageBox(NULL,"An invalid set of flags was passed in.","Error",MB_OK|MB_ICONSTOP);
                    break;
                case DISP_CHANGE_FAILED:
                    MessageBox(NULL,"The display driver failed the specified graphics mode.","Error",MB_OK|MB_ICONSTOP);
                    break;
                case DISP_CHANGE_BADMODE:
                    MessageBox(NULL,"The graphics mode is not supported.","Error",MB_OK|MB_ICONSTOP);
                case DISP_CHANGE_NOTUPDATED:
                    MessageBox(NULL,"Windows NT only: Unable to write settings to the registry.","Error",MB_OK|MB_ICONSTOP);
                    break;
                }
            ButtonTglResolution->Tag = 1;   // ---- Fullscreen mode
            break;

        // ---- Going to window mode
        case 1:
            ChangeDisplaySettings(NULL, 0);         // ---- Return to default resolution
            BorderStyle = bsSizeable;               // ---- Resizable border
            WindowState = wsNormal;                 // ---- Normal window
            SetWindowPlacement(Handle, &WindowRect);// ---- Restoring position of window
            ButtonTglResolution->Tag = 0;           // ---- Window mode
            ButtonTglResolution->Caption = "FullScreen";
            break;
        }

}
//---------------------------------------------------------------------------


